local class = require("silo.lib.class")

local Inventories = class("silo.Inventories")

function Inventories:new(ctx)
  local items = ctx:require("silo.items")
  local peripherals = ctx:require("silo.peripherals")

  self.items = items
  self.peripherals = peripherals

  ctx.bus:register("event.peripheral", function(name)
    if not self:enabled(name) then return end

    local remote = peripherals:wrap(name)
    if remote and remote.list and remote.getItemMeta then
      peripherals:execute({
        fn = function() items:loadPeripheral(name, remote) end,
        peripheral = name
      })
    end
  end)

  ctx.bus:register("event.peripheral_detach", function(name)
    items:unloadPeripheral(name)
  end)

  local function checkInventory(data)
    local name = data.name
    local inventory = items.inventories[name]
    if not inventory then return end

    items:loadPeripheral(name, inventory.remote)
  end

  ctx:addTask(function()
    local name = nil
    while true do
      sleep(0.5)

      if items.inventories[name] then
        name = next(items.inventories, name)
      else
        name = nil
      end

      if name == nil then
        name = next(items.inventories, nil)
      end

      if name ~= nil then
        peripherals:execute({
          fn = checkInventory,
          name = name,
          peripheral = name
        })
      end
    end
  end)

end

function Inventories:scan(callback)
  local items = self.items
  local peripherals = self.peripherals

  local function addInventories()
    local queue = {}
    local peripheralNames = peripheral.getNames()

    for i = 1, #peripheralNames do
      local name = peripheralNames[i]
      if self:enabled(name) then
        local remote = peripherals:wrap(name)
        if remote and remote.list and remote.getItemMeta then
          queue[#queue + 1] = function()
            items:loadPeripheral(name, remote)
          end
        end
      end
    end

    -- Note: NEVER call parallel.waitForAll() with more than, like, 4 things.
    -- SwitchCraft crashed twice.
    -- !!DANGEROUS!! if #queue > 0 then parallel.waitForAll(unpack(queue)) end

    if #queue > 0 then
      local cX, cY = term.getCursorPos()
      for i = 1, #queue do
        -- TODO: Can we do, like, two at once?
        term.setCursorPos(cX, cY)
        print("Scanning: " .. i .. " / " .. #queue)

        local f = queue[i]
        f()
      end
    end

    callback()
  end

  peripherals:execute({
    fn = addInventories,
    priority = 100,
    peripheral = true
  })
end

function Inventories:enabled(name)
  return name:match("ironchest") and not name:match("ironchest_crystal")
end

return Inventories
